@@ -49,24 +49,49 @@ struct _GnomePrinterListDetails {
 	guint new_printer_notify;
 };
 
+/* Tool to help debug lifecycle */
+#if 0
+#define d(code)	do { code } while (0)
+#else
+#define d(code)
+#endif
+
 static void disconnect_printer (GnomePrinterList *list, GnomeCupsPrinter *printer);
 
 G_DEFINE_TYPE (GnomePrinterList, gnome_printer_list, egg_icon_list_get_type ())
 
 static GdkPixbuf *
-load_icon (char const *icon_name)
+load_icon (GtkWidget *w, char const *icon_name)
 {
-	return gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-		icon_name, 48, GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
+	GError *err = NULL;
+	GdkPixbuf *res = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+		icon_name, 48, 0, NULL);
+
+	if (res == NULL) {
+		if (err != NULL) {
+			g_warning ("%s", err->message);
+			g_error_free (err);
+			err = NULL;
+		}
+		res = gtk_widget_render_icon (w, icon_name, GTK_ICON_SIZE_DIALOG, NULL);
+
+		if (err != NULL) {
+			g_warning ("%s", err->message);
+			g_error_free (err);
+			return NULL;
+		}
+	}
+
+	return res;
 }
 
 static void
 add_new_printer_icon (GnomePrinterList *list)
 {
 	EggIconListItem *item;
-	GdkPixbuf *pixbuf;
-
-	pixbuf = load_icon ("gnome-dev-printer-new");
+	GdkPixbuf *pixbuf = load_icon (GTK_WIDGET (list), "gnome-dev-printer-new");
+	if (pixbuf == NULL)
+		pixbuf = load_icon (GTK_WIDGET (list), GTK_STOCK_PRINT);
 
 	item = egg_icon_list_item_new (pixbuf, _("New Printer"));
 	list->details->add_item = item;
@@ -77,23 +102,42 @@ add_new_printer_icon (GnomePrinterList *list)
 }
 
 static GdkPixbuf *
-get_printer_icon (GnomeCupsPrinter *printer)
+get_printer_icon (GtkWidget *list, GnomeCupsPrinter *printer)
 {
 	char *icon_name;
-	GList *ptr, *emblems, *emblem_pixbufs = NULL;
-	GdkPixbuf *pixbuf;
+	GList *ptr, *emblems;
+	GdkPixbuf *pixbuf, *emblem;
+	GError *err = NULL;
 
 	gnome_cups_printer_get_icon (printer,
 				     &icon_name,
 				     &emblems);
-	pixbuf = load_icon (icon_name);
+	pixbuf = load_icon (GTK_WIDGET (list), icon_name);
 	g_free (icon_name);
-	for (ptr = emblems; ptr != NULL; ptr = ptr->next)
-		if (ptr->data) {
-			GdkPixbuf *emblem = load_icon (ptr->data);
-			if (emblem != NULL)
-				emblem_pixbufs = g_list_prepend (emblem_pixbufs, emblem);
+	if (pixbuf == NULL)
+		pixbuf = load_icon (GTK_WIDGET (list), GTK_STOCK_PRINT);
+
+	for (ptr = emblems; ptr != NULL; ptr = ptr->next) {
+		if (ptr->data == NULL)
+			continue;
+		emblem = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+			ptr->data, 48, 0, &err);
+		if (emblem == NULL) {
+			g_warning ("unable to load emblem '%s'", (char *)ptr->data);
+			if (err != NULL) {
+				g_warning ("%s", err->message);
+				g_error_free (err);
+				err = NULL;
+			}
+		} else {
+			GdkPixbuf *res = gdk_pixbuf_copy (pixbuf);
+			gdk_pixbuf_composite (emblem,  res, 0, 0, 
+				gdk_pixbuf_get_width (res), gdk_pixbuf_get_height (res), 
+				0., 0., 1., 1., GDK_INTERP_NEAREST, 255);
+			g_object_unref (pixbuf);
+			pixbuf = res;
 		}
+	}
 
 	return pixbuf;
 }
@@ -101,25 +145,28 @@ get_printer_icon (GnomeCupsPrinter *printer)
 static char *
 get_printer_label (GnomeCupsPrinter *printer)
 {
-	char *label;
-	
-	/* FIXME: hard-coded additional text color */
-	label = g_strdup_printf ("%s\n<span color=\"#33337F\">%s\n%d Jobs</span>",
-				 gnome_cups_printer_get_name (printer),
-				 gnome_cups_printer_get_state_name (printer),
-				 gnome_cups_printer_get_job_count (printer));
+	char const *name  = gnome_cups_printer_get_name (printer);
+	char const *state = gnome_cups_printer_get_state_name (printer);
+	int jobs = gnome_cups_printer_get_job_count (printer);
 
-	return label;
+	/* FIXME: hard-coded additional text color */
+	if (jobs > 0) {
+		return g_strdup_printf ("%s\n<span color=\"#337F33\">%s\n%d Jobs</span>",
+			name, state, jobs);
+	} else {
+		return g_strdup_printf ("%s\n<span color=\"#33337F\">%s\n</span>",
+			name, state);
+	}
 }
 
 static EggIconListItem *
-create_printer_item (GnomeCupsPrinter *printer)
+create_printer_item (GtkWidget *list, GnomeCupsPrinter *printer)
 {
 	EggIconListItem *item;
 	GdkPixbuf *pixbuf;
 	char *label;
 	
-	pixbuf = get_printer_icon (printer);
+	pixbuf = get_printer_icon (list, printer);
 	label = get_printer_label (printer);
 	
 	item = egg_icon_list_item_new (pixbuf, label);
@@ -345,7 +392,7 @@ create_selection_menu (GnomePrinterList *list)
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);	
 
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_MEDIA_PAUSE, NULL);
+	item = gtk_menu_item_new_with_label (_("Pause"));
 	gtk_widget_show (item);
 	gtk_widget_set_sensitive (item, pause);
 	g_signal_connect (item, "activate", 
@@ -437,17 +484,17 @@ button_press_event_cb (GnomePrinterList *list,
 
 static void
 printer_changed_cb (GnomeCupsPrinter *printer,
-		    gpointer user_data)
+		    GnomePrinterList *list)
 {
-	GnomePrinterList *list;
 	EggIconListItem *item;
 	GdkPixbuf *pixbuf;
 	char *label;
 
-	list = GNOME_PRINTER_LIST (user_data);
+	d (g_warning ("changed %s", gnome_cups_printer_get_name (printer)););
+
 	item = g_object_get_data (G_OBJECT (printer), "icon_item");
 	
-	pixbuf = get_printer_icon (printer);
+	pixbuf = get_printer_icon (GTK_WIDGET (list), printer);
 	label = get_printer_label (printer);
 	
 	egg_icon_list_item_set_icon (item, pixbuf);
@@ -466,6 +513,8 @@ printer_gone_cb (GnomeCupsPrinter *printer,
 	GnomePrinterList *list;
 	EggIconListItem *item;
 
+	d (g_warning ("removed %s", gnome_cups_printer_get_name (printer)););
+
 	list = GNOME_PRINTER_LIST (user_data);
 	item = g_object_get_data (G_OBJECT (printer), "icon_item");
 
@@ -493,20 +542,19 @@ static void
 watch_printer (GnomePrinterList *list,
 	       const char *printer_name)
 {
-	GnomeCupsPrinter *printer;
 	EggIconListItem *item;
-	
-	printer = gnome_cups_printer_get (printer_name);
-	if (!printer) {
+	GnomeCupsPrinter *printer = gnome_cups_printer_get (printer_name);
+
+	if (!printer)
 		return;
-	}
-	
+
 	list->details->printers = g_list_append (list->details->printers, 
 						 printer);
 	g_object_ref (printer);
 	
-	item = create_printer_item (printer);
-	g_object_set_data (G_OBJECT (printer), "icon_item", item);
+	item = create_printer_item (GTK_WIDGET (list), printer);
+	g_object_set_data_full (G_OBJECT (printer), "icon_item", item,
+		(GDestroyNotify)egg_icon_list_item_unref);
 
 	egg_icon_list_append_item (EGG_ICON_LIST (list), item);
 
@@ -519,6 +567,7 @@ watch_printer (GnomePrinterList *list,
 	g_signal_connect_object (printer, "gone",
 				 G_CALLBACK (printer_gone_cb),
 				 list, 0);
+	printer_changed_cb (printer, list);
 }
 
 static void
@@ -528,6 +577,7 @@ printer_added_cb (const char *name, gpointer user_data)
 	
 	list = GNOME_PRINTER_LIST (user_data);
 	
+	d (g_warning ("added %s", name););
 	watch_printer (list, name);
 }
 
@@ -563,8 +613,10 @@ icon_compare (EggIconList *icon_list,
 void
 gnome_printer_list_new_printer (GnomePrinterList *list)
 {
-	gnome_cups_spawn ("gnome-cups-add", 0, NULL,
-		TRUE, gtk_widget_get_toplevel (GTK_WIDGET (list)));
+	GtkWidget *toplevel = NULL;
+	if (list != NULL)
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (list));
+	gnome_cups_spawn ("gnome-cups-add", 0, NULL, TRUE, toplevel);
 }
 
 void
